# -------------------------------------------------------------------------------------------
#
# How to build for SMP and UP kernel
#
#  rpmbuild -ba --target i686   --define 'kernel 2.6.9-55.0.2.EL' unionfs.spec
#  rpmbuild -ba --target x86_64 --define 'kernel 2.6.9-55.0.2.EL' unionfs.spec
#
# or only for UP kernel
#
#  rpmbuild -ba --target i686   --define 'kernel 2.6.9-55.0.2.EL' --define 'buildsmp 0' unionfs.spec
#  rpmbuild -ba --target x86_64 --define 'kernel 2.6.9-55.0.2.EL' --define 'buildsmp 0' unionfs.spec
#
#
# for the running kernel:
#
#  rpmbuild -ba --target i686   unionfs.spec
#  rpmbuild -ba --target x86_64 unionfs.spec
#
# -------------------------------------------------------------------------------------------

%{!?buildsmp:%define buildsmp 1}

%{?!kernel:%define kernel %(rpm -q kernel-devel | tail -1 | sed -e 's|kernel-devel-||')}

%define kversion %(echo "%{kernel}" | sed -e 's|-.*||')
%define krelease %(echo "%{kernel}" | sed -e 's|.*-||')

%define kernel_src_path %{_usrsrc}/kernels/%{kernel}-%{_target_cpu}
%define kernel_smp_src_path %{_usrsrc}/kernels/%{kernel}-smp-%{_target_cpu}

%define kernel_moduledir /lib/modules/%{kernel}
%define kernel_smp_moduledir /lib/modules/%{kernel}smp

%define pkg_name unionfs
%define kmdl_name kernel-module-%{pkg_name}-%{kernel}

# -------------------------------------------------------------------------------------------

%define PATCH_NAME unionfs-2.1.3_for_2.6.9-final.diff

Summary: 	A Stackable Unification File System
Name: 		%{pkg_name}
Version: 	2.1.3
Release: 	6%{dist}
Epoch: 		0
Packager: 	Urs Beyerle <urs.beyerle@psi.ch>
License: 	GPL
Group: 		System Environment/Kernel
Source0: 	http://download.filesystems.org/unionfs/unionfs-2.1/%{PATCH_NAME}.gz
Source1:	linux-2.6.9-fake.tar.gz
Patch0:		unionfs-2.0-compat.h.patch
Patch1:		unionfs-2.1.2-Makefile.patch
Patch2:		unionfs-2.1.2-super.c.livecd.patch
URL: 		http://www.am-utils.org/project-unionfs.html
BuildRoot: 	%{_tmppath}/%{name}-%{version}-%{release}-buildroot
Requires:       kernel-module-%{pkg_name} >= %{version}-%{release}
BuildRequires: 	kernel-devel = %{kernel}, 
%if %{buildsmp}
BuildRequires: 	kernel-smp-devel = %{kernel}
%endif

%description
This project builds a stackable unification file system, which 
can appear to merge the contents of several directories (branches), 
while keeping their physical content separate. Unionfs is useful 
for unified source tree management, merged contents of split 
CD-ROM, merged separate software package directories, data grids, 
and more. Unionfs allows any mix of read-only and read-write 
branches, as well as insertion and deletion of branches anywhere 
in the fan-out. To maintain unix semantics, Unionfs handles 
elimination of duplicates, partial-error conditions, and more. 
Unionfs is part of the larger FiST project.

Kernel modules for unionfs are in kernel-module-%{pkg_name} rpms.

%package -n 	%{kmdl_name}
Summary:        kernel modules for %{name}.
Group:          System Environment/Kernel
License:        GPL
Provides:       kernel-module-%{pkg_name} = %{epoch}:%{version}-%{release}
# Requires:       %{pkg_name}

%description -n %{kmdl_name}
Kernel modules for %{name}.

%if %{buildsmp}

%package -n 	%{kmdl_name}smp
Summary: 	smp kernel modules for %{name}.
Group: 		System Environment/Kernel
License: 	GPL
Provides:	kernel-module-%{pkg_name} = %{epoch}:%{version}-%{release}
# Requires:	%{pkg_name}

%description -n %{kmdl_name}smp
SMP kernel modules for %{name}.

%endif

%prep
# Since the unionfs source is delivered as patch against 2.6.9 kernel
# we have to unpack it quite complicate
rm -rf %{name}-%{version}
mkdir -p %{name}-%{version}

# get source files
cp -a %{SOURCE1} %{name}-%{version}/
cp -a %{SOURCE0} %{name}-%{version}/
cd %{name}-%{version}

# create a fake linux source tree
tar xfvz %{SOURCE1}

# apply unionfs patch
gunzip %{PATCH_NAME}.gz
patch -d linux-2.6.9-fake -p1 < %{PATCH_NAME}

# patches for RHEL4 kernel
cd linux-2.6.9-fake
%patch0 -p1
%patch1 -p0

# do not exit, if flags other than ro,rw are set 
# - just print kernel warning
%patch2 -p0

# copy stack.c inside fs/unionfs
cp -a fs/stack.c fs/unionfs/


%build
cd %{name}-%{version}/linux-2.6.9-fake
echo -e "\nDriver version: %{version}\nKernel version: %{kernel}\n"

### UP kernel
make -C %{kernel_moduledir}/build SUBDIRS=$PWD/fs/unionfs clean
EXTRA_CFLAGS="-I$PWD/include" \
  make -C %{kernel_moduledir}/build \
  CONFIG_UNION_FS=m \
  SUBDIRS=$PWD/fs/unionfs modules
# CONFIG_UNIONFS_DEBUG=y \

INST_MODLIB="${RPM_BUILD_ROOT}/lib/modules/%{kernel}/kernel"
mkdir -p "${INST_MODLIB}/fs/unionfs"
install -p -m 744 fs/unionfs/unionfs.ko "${INST_MODLIB}/fs/unionfs/"

### SMP kernel
%if %{buildsmp}

make -C %{kernel_smp_moduledir}/build CONFIG_UNION_FS=m SUBDIRS=$PWD/fs/unionfs clean
EXTRA_CFLAGS="-I$PWD/include" \
   make -C %{kernel_smp_moduledir}/build \
   CONFIG_UNION_FS=m \
   SUBDIRS=$PWD/fs/unionfs modules

INST_MODLIB="${RPM_BUILD_ROOT}/lib/modules/%{kernel}smp/kernel"
mkdir -p "${INST_MODLIB}/fs/unionfs"
install -p -m 744 fs/unionfs/unionfs.ko "${INST_MODLIB}/fs/unionfs/"

%endif


%install
echo 


%clean
[ "$RPM_BUILD_ROOT" != "/" ] && rm -rf $RPM_BUILD_ROOT


%postun -n %{kmdl_name}
depmod -a %{kernel} >/dev/null 2>&1 || :	

%post -n %{kmdl_name}
depmod -a %{kernel} >/dev/null 2>&1 || :

%if %{buildsmp}
%postun -n %{kmdl_name}smp
depmod -a %{kernel}smp >/dev/null 2>&1 || :

%post -n %{kmdl_name}smp
depmod -a %{kernel}smp >/dev/null 2>&1 || :
%endif

%files 
%defattr(-,root,root,-)
%doc %{name}-%{version}/linux-2.6.9-fake/Documentation/filesystems/unionfs/*

%files -n %{kmdl_name}
%defattr(-,root,root,-)
/lib/modules/%{kernel}/kernel/fs/unionfs

%if %{buildsmp}
%files -n %{kmdl_name}smp
%defattr(-,root,root,-)
/lib/modules/%{kernel}smp/kernel/fs/unionfs
%endif

%changelog
* Tue Sep 18 2007 Urs Beyerle <urs.beyerle@psi.ch>  2.1.3-6
- update to 2.1.3

* Wed Aug 29 2007 Urs Beyerle <urs.beyerle@psi.ch>  2.1.2-6
- update to 2.1.2

* Thu Aug 16 2007 Urs Beyerle <urs.beyerle@psi.ch>  2.1-5
- add unionfs-2.1-super.c.livecd.patch
  removes check for remount flags in super.c, fixes a problem
  with busybox mount command, which remount with flag 0x8000

* Wed Aug 15 2007 Urs Beyerle <urs.beyerle@psi.ch>  2.1-4
- add unionfs-2.1-super.c.patch

* Thu Aug 09 2007 Urs Beyerle <urs.beyerle@psi.ch>  2.1-3
- update to 2.1
- remove unionfs-2.0-compat.c.patch

* Mon Aug 06 2007 Urs Beyerle <urs.beyerle@psi.ch>  2.0-2
- update to 2.0 !
- compile stack.c inside fs/unionfs
- kernel-module-unionfs does not requires unionfs package, which only
  contains documentation

* Mon Oct 16 2006 Urs Beyerle <urs.beyerle@psi.ch>  1.1.5-2.sl4.psi
- add unionctl.static (statically linked unionctl), needed for LiveCD

* Thu Aug 31 2006 Urs Beyerle <urs.beyerle@psi.ch>  1.1.5-1.sl4.psi
- inital build.
